@@ -4,7 +4,7 @@
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
 //
-//   http://www.apache.org/licenses/LICENSE-2.0
+//   https://www.apache.org/licenses/LICENSE-2.0
 //
 //   Unless required by applicable law or agreed to in writing, software
 //   distributed under the License is distributed on an "AS IS" BASIS,
@@ -20,8 +20,8 @@
 #include "cctz/time_zone.h"
 
 template <typename D>
-cctz::time_point<cctz::sys_seconds> FloorDay(cctz::time_point<D> tp,
-                                             cctz::time_zone tz) {
+cctz::time_point<cctz::seconds> FloorDay(cctz::time_point<D> tp,
+                                         cctz::time_zone tz) {
   return cctz::convert(cctz::civil_day(cctz::convert(tp, tz)), tz);
 }
 
