@@ -45,8 +45,58 @@ void BM_Step_Days(benchmark::State& state) {
 }
 BENCHMARK(BM_Step_Days);
 
-const char RFC3339_full[] = "%Y-%m-%dT%H:%M:%E*S%Ez";
-const char RFC3339_sec[] = "%Y-%m-%dT%H:%M:%S%Ez";
+void BM_GetWeekday(benchmark::State& state) {
+  const cctz::civil_day c(2014, 8, 22);
+  while (state.KeepRunning()) {
+    benchmark::DoNotOptimize(cctz::get_weekday(c));
+  }
+}
+BENCHMARK(BM_GetWeekday);
+
+void BM_NextWeekday(benchmark::State& state) {
+  const cctz::civil_day kStart(2014, 8, 22);
+  const cctz::civil_day kDays[7] = {
+      kStart + 0, kStart + 1, kStart + 2, kStart + 3,
+      kStart + 4, kStart + 5, kStart + 6,
+  };
+  const cctz::weekday kWeekdays[7] = {
+      cctz::weekday::monday,   cctz::weekday::tuesday, cctz::weekday::wednesday,
+      cctz::weekday::thursday, cctz::weekday::friday,  cctz::weekday::saturday,
+      cctz::weekday::sunday,
+  };
+  while (state.KeepRunningBatch(7 * 7)) {
+    for (const auto from : kDays) {
+      for (const auto to : kWeekdays) {
+        benchmark::DoNotOptimize(cctz::next_weekday(from, to));
+      }
+    }
+  }
+}
+BENCHMARK(BM_NextWeekday);
+
+void BM_PrevWeekday(benchmark::State& state) {
+  const cctz::civil_day kStart(2014, 8, 22);
+  const cctz::civil_day kDays[7] = {
+      kStart + 0, kStart + 1, kStart + 2, kStart + 3,
+      kStart + 4, kStart + 5, kStart + 6,
+  };
+  const cctz::weekday kWeekdays[7] = {
+      cctz::weekday::monday,   cctz::weekday::tuesday, cctz::weekday::wednesday,
+      cctz::weekday::thursday, cctz::weekday::friday,  cctz::weekday::saturday,
+      cctz::weekday::sunday,
+  };
+  while (state.KeepRunningBatch(7 * 7)) {
+    for (const auto from : kDays) {
+      for (const auto to : kWeekdays) {
+        benchmark::DoNotOptimize(cctz::prev_weekday(from, to));
+      }
+    }
+  }
+}
+BENCHMARK(BM_PrevWeekday);
+
+const char RFC3339_full[] = "%Y-%m-%d%ET%H:%M:%E*S%Ez";
+const char RFC3339_sec[] = "%Y-%m-%d%ET%H:%M:%S%Ez";
 
 const char RFC1123_full[] = "%a, %d %b %Y %H:%M:%S %z";
 const char RFC1123_no_wday[] = "%d %b %Y %H:%M:%S %z";
@@ -59,7 +109,6 @@ const char* const kTimeZoneNames[] = {
   "Africa/Addis_Ababa",
   "Africa/Algiers",
   "Africa/Asmara",
-  "Africa/Asmera",
   "Africa/Bamako",
   "Africa/Bangui",
   "Africa/Banjul",
@@ -115,7 +164,6 @@ const char* const kTimeZoneNames[] = {
   "America/Araguaina",
   "America/Argentina/Buenos_Aires",
   "America/Argentina/Catamarca",
-  "America/Argentina/ComodRivadavia",
   "America/Argentina/Cordoba",
   "America/Argentina/Jujuy",
   "America/Argentina/La_Rioja",
@@ -139,18 +187,16 @@ const char* const kTimeZoneNames[] = {
   "America/Boa_Vista",
   "America/Bogota",
   "America/Boise",
-  "America/Buenos_Aires",
   "America/Cambridge_Bay",
   "America/Campo_Grande",
   "America/Cancun",
   "America/Caracas",
-  "America/Catamarca",
   "America/Cayenne",
   "America/Cayman",
   "America/Chicago",
   "America/Chihuahua",
+  "America/Ciudad_Juarez",
   "America/Coral_Harbour",
-  "America/Cordoba",
   "America/Costa_Rica",
   "America/Creston",
   "America/Cuiaba",
@@ -166,7 +212,6 @@ const char* const kTimeZoneNames[] = {
   "America/El_Salvador",
   "America/Ensenada",
   "America/Fort_Nelson",
-  "America/Fort_Wayne",
   "America/Fortaleza",
   "America/Glace_Bay",
   "America/Godthab",
@@ -188,20 +233,16 @@ const char* const kTimeZoneNames[] = {
   "America/Indiana/Vevay",
   "America/Indiana/Vincennes",
   "America/Indiana/Winamac",
-  "America/Indianapolis",
   "America/Inuvik",
   "America/Iqaluit",
   "America/Jamaica",
-  "America/Jujuy",
   "America/Juneau",
   "America/Kentucky/Louisville",
   "America/Kentucky/Monticello",
-  "America/Knox_IN",
   "America/Kralendijk",
   "America/La_Paz",
   "America/Lima",
   "America/Los_Angeles",
-  "America/Louisville",
   "America/Lower_Princes",
   "America/Maceio",
   "America/Managua",
@@ -210,7 +251,6 @@ const char* const kTimeZoneNames[] = {
   "America/Martinique",
   "America/Matamoros",
   "America/Mazatlan",
-  "America/Mendoza",
   "America/Menominee",
   "America/Merida",
   "America/Metlakatla",
@@ -229,6 +269,7 @@ const char* const kTimeZoneNames[] = {
   "America/North_Dakota/Beulah",
   "America/North_Dakota/Center",
   "America/North_Dakota/New_Salem",
+  "America/Nuuk",
   "America/Ojinaga",
   "America/Panama",
   "America/Pangnirtung",
@@ -246,7 +287,6 @@ const char* const kTimeZoneNames[] = {
   "America/Regina",
   "America/Resolute",
   "America/Rio_Branco",
-  "America/Rosario",
   "America/Santa_Isabel",
   "America/Santarem",
   "America/Santiago",
@@ -282,7 +322,6 @@ const char* const kTimeZoneNames[] = {
   "Antarctica/McMurdo",
   "Antarctica/Palmer",
   "Antarctica/Rothera",
-  "Antarctica/South_Pole",
   "Antarctica/Syowa",
   "Antarctica/Troll",
   "Antarctica/Vostok",
@@ -294,7 +333,6 @@ const char* const kTimeZoneNames[] = {
   "Asia/Aqtau",
   "Asia/Aqtobe",
   "Asia/Ashgabat",
-  "Asia/Ashkhabad",
   "Asia/Atyrau",
   "Asia/Baghdad",
   "Asia/Bahrain",
@@ -304,13 +342,10 @@ const char* const kTimeZoneNames[] = {
   "Asia/Beirut",
   "Asia/Bishkek",
   "Asia/Brunei",
-  "Asia/Calcutta",
   "Asia/Chita",
   "Asia/Choibalsan",
   "Asia/Chongqing",
-  "Asia/Chungking",
   "Asia/Colombo",
-  "Asia/Dacca",
   "Asia/Damascus",
   "Asia/Dhaka",
   "Asia/Dili",
@@ -333,14 +368,12 @@ const char* const kTimeZoneNames[] = {
   "Asia/Karachi",
   "Asia/Kashgar",
   "Asia/Kathmandu",
-  "Asia/Katmandu",
   "Asia/Khandyga",
   "Asia/Kolkata",
   "Asia/Krasnoyarsk",
   "Asia/Kuala_Lumpur",
   "Asia/Kuching",
   "Asia/Kuwait",
-  "Asia/Macao",
   "Asia/Macau",
   "Asia/Magadan",
   "Asia/Makassar",
@@ -357,9 +390,7 @@ const char* const kTimeZoneNames[] = {
   "Asia/Qatar",
   "Asia/Qostanay",
   "Asia/Qyzylorda",
-  "Asia/Rangoon",
   "Asia/Riyadh",
-  "Asia/Saigon",
   "Asia/Sakhalin",
   "Asia/Samarkand",
   "Asia/Seoul",
@@ -371,13 +402,10 @@ const char* const kTimeZoneNames[] = {
   "Asia/Tbilisi",
   "Asia/Tehran",
   "Asia/Tel_Aviv",
-  "Asia/Thimbu",
   "Asia/Thimphu",
   "Asia/Tokyo",
   "Asia/Tomsk",
-  "Asia/Ujung_Pandang",
   "Asia/Ulaanbaatar",
-  "Asia/Ulan_Bator",
   "Asia/Urumqi",
   "Asia/Ust-Nera",
   "Asia/Vientiane",
@@ -390,7 +418,6 @@ const char* const kTimeZoneNames[] = {
   "Atlantic/Bermuda",
   "Atlantic/Canary",
   "Atlantic/Cape_Verde",
-  "Atlantic/Faeroe",
   "Atlantic/Faroe",
   "Atlantic/Jan_Mayen",
   "Atlantic/Madeira",
@@ -398,7 +425,6 @@ const char* const kTimeZoneNames[] = {
   "Atlantic/South_Georgia",
   "Atlantic/St_Helena",
   "Atlantic/Stanley",
-  "Australia/ACT",
   "Australia/Adelaide",
   "Australia/Brisbane",
   "Australia/Broken_Hill",
@@ -407,42 +433,12 @@ const char* const kTimeZoneNames[] = {
   "Australia/Darwin",
   "Australia/Eucla",
   "Australia/Hobart",
-  "Australia/LHI",
   "Australia/Lindeman",
   "Australia/Lord_Howe",
   "Australia/Melbourne",
-  "Australia/NSW",
-  "Australia/North",
   "Australia/Perth",
-  "Australia/Queensland",
-  "Australia/South",
   "Australia/Sydney",
-  "Australia/Tasmania",
-  "Australia/Victoria",
-  "Australia/West",
   "Australia/Yancowinna",
-  "Brazil/Acre",
-  "Brazil/DeNoronha",
-  "Brazil/East",
-  "Brazil/West",
-  "CET",
-  "CST6CDT",
-  "Canada/Atlantic",
-  "Canada/Central",
-  "Canada/Eastern",
-  "Canada/Mountain",
-  "Canada/Newfoundland",
-  "Canada/Pacific",
-  "Canada/Saskatchewan",
-  "Canada/Yukon",
-  "Chile/Continental",
-  "Chile/EasterIsland",
-  "Cuba",
-  "EET",
-  "EST",
-  "EST5EDT",
-  "Egypt",
-  "Eire",
   "Etc/GMT",
   "Etc/GMT+0",
   "Etc/GMT+1",
@@ -500,8 +496,8 @@ const char* const kTimeZoneNames[] = {
   "Europe/Istanbul",
   "Europe/Jersey",
   "Europe/Kaliningrad",
-  "Europe/Kiev",
   "Europe/Kirov",
+  "Europe/Kyiv",
   "Europe/Lisbon",
   "Europe/Ljubljana",
   "Europe/London",
@@ -531,7 +527,6 @@ const char* const kTimeZoneNames[] = {
   "Europe/Tirane",
   "Europe/Tiraspol",
   "Europe/Ulyanovsk",
-  "Europe/Uzhgorod",
   "Europe/Vaduz",
   "Europe/Vatican",
   "Europe/Vienna",
@@ -539,18 +534,8 @@ const char* const kTimeZoneNames[] = {
   "Europe/Volgograd",
   "Europe/Warsaw",
   "Europe/Zagreb",
-  "Europe/Zaporozhye",
   "Europe/Zurich",
-  "GB",
-  "GB-Eire",
-  "GMT",
-  "GMT+0",
-  "GMT-0",
-  "GMT0",
-  "Greenwich",
-  "HST",
-  "Hongkong",
-  "Iceland",
+  "Factory",
   "Indian/Antananarivo",
   "Indian/Chagos",
   "Indian/Christmas",
@@ -562,23 +547,6 @@ const char* const kTimeZoneNames[] = {
   "Indian/Mauritius",
   "Indian/Mayotte",
   "Indian/Reunion",
-  "Iran",
-  "Israel",
-  "Jamaica",
-  "Japan",
-  "Kwajalein",
-  "Libya",
-  "MET",
-  "MST",
-  "MST7MDT",
-  "Mexico/BajaNorte",
-  "Mexico/BajaSur",
-  "Mexico/General",
-  "NZ",
-  "NZ-CHAT",
-  "Navajo",
-  "PRC",
-  "PST8PDT",
   "Pacific/Apia",
   "Pacific/Auckland",
   "Pacific/Bougainville",
@@ -586,7 +554,6 @@ const char* const kTimeZoneNames[] = {
   "Pacific/Chuuk",
   "Pacific/Easter",
   "Pacific/Efate",
-  "Pacific/Enderbury",
   "Pacific/Fakaofo",
   "Pacific/Fiji",
   "Pacific/Funafuti",
@@ -596,6 +563,7 @@ const char* const kTimeZoneNames[] = {
   "Pacific/Guam",
   "Pacific/Honolulu",
   "Pacific/Johnston",
+  "Pacific/Kanton",
   "Pacific/Kiritimati",
   "Pacific/Kosrae",
   "Pacific/Kwajalein",
@@ -610,7 +578,6 @@ const char* const kTimeZoneNames[] = {
   "Pacific/Palau",
   "Pacific/Pitcairn",
   "Pacific/Pohnpei",
-  "Pacific/Ponape",
   "Pacific/Port_Moresby",
   "Pacific/Rarotonga",
   "Pacific/Saipan",
@@ -618,34 +585,10 @@ const char* const kTimeZoneNames[] = {
   "Pacific/Tahiti",
   "Pacific/Tarawa",
   "Pacific/Tongatapu",
-  "Pacific/Truk",
   "Pacific/Wake",
   "Pacific/Wallis",
   "Pacific/Yap",
-  "Poland",
-  "Portugal",
-  "ROC",
-  "ROK",
-  "Singapore",
-  "Turkey",
-  "UCT",
-  "US/Alaska",
-  "US/Aleutian",
-  "US/Arizona",
-  "US/Central",
-  "US/East-Indiana",
-  "US/Eastern",
-  "US/Hawaii",
-  "US/Indiana-Starke",
-  "US/Michigan",
-  "US/Mountain",
-  "US/Pacific",
-  "US/Samoa",
   "UTC",
-  "Universal",
-  "W-SU",
-  "WET",
-  "Zulu",
   nullptr
 };
 
@@ -940,12 +883,12 @@ void BM_Time_FromCivilDay0_Libc(benchmark::State& state) {
 BENCHMARK(BM_Time_FromCivilDay0_Libc);
 
 const char* const kFormats[] = {
-    RFC1123_full,         // 0
-    RFC1123_no_wday,      // 1
-    RFC3339_full,         // 2
-    RFC3339_sec,          // 3
-    "%Y-%m-%dT%H:%M:%S",  // 4
-    "%Y-%m-%d",           // 5
+    RFC1123_full,           // 0
+    RFC1123_no_wday,        // 1
+    RFC3339_full,           // 2
+    RFC3339_sec,            // 3
+    "%Y-%m-%d%ET%H:%M:%S",  // 4
+    "%Y-%m-%d",             // 5
 };
 const int kNumFormats = sizeof(kFormats) / sizeof(kFormats[0]);
 
