@@ -22,6 +22,9 @@
 #include <string>
 #include <thread>
 #include <vector>
+#if defined(__linux__)
+#include <features.h>
+#endif
 
 #include "cctz/civil_time.h"
 #include "gtest/gtest.h"
@@ -39,7 +42,6 @@ const char* const kTimeZoneNames[] = {
   "Africa/Addis_Ababa",
   "Africa/Algiers",
   "Africa/Asmara",
-  "Africa/Asmera",
   "Africa/Bamako",
   "Africa/Bangui",
   "Africa/Banjul",
@@ -95,7 +97,6 @@ const char* const kTimeZoneNames[] = {
   "America/Araguaina",
   "America/Argentina/Buenos_Aires",
   "America/Argentina/Catamarca",
-  "America/Argentina/ComodRivadavia",
   "America/Argentina/Cordoba",
   "America/Argentina/Jujuy",
   "America/Argentina/La_Rioja",
@@ -119,18 +120,16 @@ const char* const kTimeZoneNames[] = {
   "America/Boa_Vista",
   "America/Bogota",
   "America/Boise",
-  "America/Buenos_Aires",
   "America/Cambridge_Bay",
   "America/Campo_Grande",
   "America/Cancun",
   "America/Caracas",
-  "America/Catamarca",
   "America/Cayenne",
   "America/Cayman",
   "America/Chicago",
   "America/Chihuahua",
+  "America/Ciudad_Juarez",
   "America/Coral_Harbour",
-  "America/Cordoba",
   "America/Costa_Rica",
   "America/Creston",
   "America/Cuiaba",
@@ -146,7 +145,6 @@ const char* const kTimeZoneNames[] = {
   "America/El_Salvador",
   "America/Ensenada",
   "America/Fort_Nelson",
-  "America/Fort_Wayne",
   "America/Fortaleza",
   "America/Glace_Bay",
   "America/Godthab",
@@ -168,20 +166,16 @@ const char* const kTimeZoneNames[] = {
   "America/Indiana/Vevay",
   "America/Indiana/Vincennes",
   "America/Indiana/Winamac",
-  "America/Indianapolis",
   "America/Inuvik",
   "America/Iqaluit",
   "America/Jamaica",
-  "America/Jujuy",
   "America/Juneau",
   "America/Kentucky/Louisville",
   "America/Kentucky/Monticello",
-  "America/Knox_IN",
   "America/Kralendijk",
   "America/La_Paz",
   "America/Lima",
   "America/Los_Angeles",
-  "America/Louisville",
   "America/Lower_Princes",
   "America/Maceio",
   "America/Managua",
@@ -190,7 +184,6 @@ const char* const kTimeZoneNames[] = {
   "America/Martinique",
   "America/Matamoros",
   "America/Mazatlan",
-  "America/Mendoza",
   "America/Menominee",
   "America/Merida",
   "America/Metlakatla",
@@ -209,6 +202,7 @@ const char* const kTimeZoneNames[] = {
   "America/North_Dakota/Beulah",
   "America/North_Dakota/Center",
   "America/North_Dakota/New_Salem",
+  "America/Nuuk",
   "America/Ojinaga",
   "America/Panama",
   "America/Pangnirtung",
@@ -226,7 +220,6 @@ const char* const kTimeZoneNames[] = {
   "America/Regina",
   "America/Resolute",
   "America/Rio_Branco",
-  "America/Rosario",
   "America/Santa_Isabel",
   "America/Santarem",
   "America/Santiago",
@@ -262,7 +255,6 @@ const char* const kTimeZoneNames[] = {
   "Antarctica/McMurdo",
   "Antarctica/Palmer",
   "Antarctica/Rothera",
-  "Antarctica/South_Pole",
   "Antarctica/Syowa",
   "Antarctica/Troll",
   "Antarctica/Vostok",
@@ -274,7 +266,6 @@ const char* const kTimeZoneNames[] = {
   "Asia/Aqtau",
   "Asia/Aqtobe",
   "Asia/Ashgabat",
-  "Asia/Ashkhabad",
   "Asia/Atyrau",
   "Asia/Baghdad",
   "Asia/Bahrain",
@@ -284,13 +275,10 @@ const char* const kTimeZoneNames[] = {
   "Asia/Beirut",
   "Asia/Bishkek",
   "Asia/Brunei",
-  "Asia/Calcutta",
   "Asia/Chita",
   "Asia/Choibalsan",
   "Asia/Chongqing",
-  "Asia/Chungking",
   "Asia/Colombo",
-  "Asia/Dacca",
   "Asia/Damascus",
   "Asia/Dhaka",
   "Asia/Dili",
@@ -313,14 +301,12 @@ const char* const kTimeZoneNames[] = {
   "Asia/Karachi",
   "Asia/Kashgar",
   "Asia/Kathmandu",
-  "Asia/Katmandu",
   "Asia/Khandyga",
   "Asia/Kolkata",
   "Asia/Krasnoyarsk",
   "Asia/Kuala_Lumpur",
   "Asia/Kuching",
   "Asia/Kuwait",
-  "Asia/Macao",
   "Asia/Macau",
   "Asia/Magadan",
   "Asia/Makassar",
@@ -337,9 +323,7 @@ const char* const kTimeZoneNames[] = {
   "Asia/Qatar",
   "Asia/Qostanay",
   "Asia/Qyzylorda",
-  "Asia/Rangoon",
   "Asia/Riyadh",
-  "Asia/Saigon",
   "Asia/Sakhalin",
   "Asia/Samarkand",
   "Asia/Seoul",
@@ -351,13 +335,10 @@ const char* const kTimeZoneNames[] = {
   "Asia/Tbilisi",
   "Asia/Tehran",
   "Asia/Tel_Aviv",
-  "Asia/Thimbu",
   "Asia/Thimphu",
   "Asia/Tokyo",
   "Asia/Tomsk",
-  "Asia/Ujung_Pandang",
   "Asia/Ulaanbaatar",
-  "Asia/Ulan_Bator",
   "Asia/Urumqi",
   "Asia/Ust-Nera",
   "Asia/Vientiane",
@@ -370,7 +351,6 @@ const char* const kTimeZoneNames[] = {
   "Atlantic/Bermuda",
   "Atlantic/Canary",
   "Atlantic/Cape_Verde",
-  "Atlantic/Faeroe",
   "Atlantic/Faroe",
   "Atlantic/Jan_Mayen",
   "Atlantic/Madeira",
@@ -378,7 +358,6 @@ const char* const kTimeZoneNames[] = {
   "Atlantic/South_Georgia",
   "Atlantic/St_Helena",
   "Atlantic/Stanley",
-  "Australia/ACT",
   "Australia/Adelaide",
   "Australia/Brisbane",
   "Australia/Broken_Hill",
@@ -387,42 +366,12 @@ const char* const kTimeZoneNames[] = {
   "Australia/Darwin",
   "Australia/Eucla",
   "Australia/Hobart",
-  "Australia/LHI",
   "Australia/Lindeman",
   "Australia/Lord_Howe",
   "Australia/Melbourne",
-  "Australia/NSW",
-  "Australia/North",
   "Australia/Perth",
-  "Australia/Queensland",
-  "Australia/South",
   "Australia/Sydney",
-  "Australia/Tasmania",
-  "Australia/Victoria",
-  "Australia/West",
   "Australia/Yancowinna",
-  "Brazil/Acre",
-  "Brazil/DeNoronha",
-  "Brazil/East",
-  "Brazil/West",
-  "CET",
-  "CST6CDT",
-  "Canada/Atlantic",
-  "Canada/Central",
-  "Canada/Eastern",
-  "Canada/Mountain",
-  "Canada/Newfoundland",
-  "Canada/Pacific",
-  "Canada/Saskatchewan",
-  "Canada/Yukon",
-  "Chile/Continental",
-  "Chile/EasterIsland",
-  "Cuba",
-  "EET",
-  "EST",
-  "EST5EDT",
-  "Egypt",
-  "Eire",
   "Etc/GMT",
   "Etc/GMT+0",
   "Etc/GMT+1",
@@ -480,8 +429,8 @@ const char* const kTimeZoneNames[] = {
   "Europe/Istanbul",
   "Europe/Jersey",
   "Europe/Kaliningrad",
-  "Europe/Kiev",
   "Europe/Kirov",
+  "Europe/Kyiv",
   "Europe/Lisbon",
   "Europe/Ljubljana",
   "Europe/London",
@@ -511,7 +460,6 @@ const char* const kTimeZoneNames[] = {
   "Europe/Tirane",
   "Europe/Tiraspol",
   "Europe/Ulyanovsk",
-  "Europe/Uzhgorod",
   "Europe/Vaduz",
   "Europe/Vatican",
   "Europe/Vienna",
@@ -519,18 +467,8 @@ const char* const kTimeZoneNames[] = {
   "Europe/Volgograd",
   "Europe/Warsaw",
   "Europe/Zagreb",
-  "Europe/Zaporozhye",
   "Europe/Zurich",
-  "GB",
-  "GB-Eire",
-  "GMT",
-  "GMT+0",
-  "GMT-0",
-  "GMT0",
-  "Greenwich",
-  "HST",
-  "Hongkong",
-  "Iceland",
+  "Factory",
   "Indian/Antananarivo",
   "Indian/Chagos",
   "Indian/Christmas",
@@ -542,23 +480,6 @@ const char* const kTimeZoneNames[] = {
   "Indian/Mauritius",
   "Indian/Mayotte",
   "Indian/Reunion",
-  "Iran",
-  "Israel",
-  "Jamaica",
-  "Japan",
-  "Kwajalein",
-  "Libya",
-  "MET",
-  "MST",
-  "MST7MDT",
-  "Mexico/BajaNorte",
-  "Mexico/BajaSur",
-  "Mexico/General",
-  "NZ",
-  "NZ-CHAT",
-  "Navajo",
-  "PRC",
-  "PST8PDT",
   "Pacific/Apia",
   "Pacific/Auckland",
   "Pacific/Bougainville",
@@ -566,7 +487,6 @@ const char* const kTimeZoneNames[] = {
   "Pacific/Chuuk",
   "Pacific/Easter",
   "Pacific/Efate",
-  "Pacific/Enderbury",
   "Pacific/Fakaofo",
   "Pacific/Fiji",
   "Pacific/Funafuti",
@@ -576,6 +496,7 @@ const char* const kTimeZoneNames[] = {
   "Pacific/Guam",
   "Pacific/Honolulu",
   "Pacific/Johnston",
+  "Pacific/Kanton",
   "Pacific/Kiritimati",
   "Pacific/Kosrae",
   "Pacific/Kwajalein",
@@ -590,7 +511,6 @@ const char* const kTimeZoneNames[] = {
   "Pacific/Palau",
   "Pacific/Pitcairn",
   "Pacific/Pohnpei",
-  "Pacific/Ponape",
   "Pacific/Port_Moresby",
   "Pacific/Rarotonga",
   "Pacific/Saipan",
@@ -598,34 +518,10 @@ const char* const kTimeZoneNames[] = {
   "Pacific/Tahiti",
   "Pacific/Tarawa",
   "Pacific/Tongatapu",
-  "Pacific/Truk",
   "Pacific/Wake",
   "Pacific/Wallis",
   "Pacific/Yap",
-  "Poland",
-  "Portugal",
-  "ROC",
-  "ROK",
-  "Singapore",
-  "Turkey",
-  "UCT",
-  "US/Alaska",
-  "US/Aleutian",
-  "US/Arizona",
-  "US/Central",
-  "US/East-Indiana",
-  "US/Eastern",
-  "US/Hawaii",
-  "US/Indiana-Starke",
-  "US/Michigan",
-  "US/Mountain",
-  "US/Pacific",
-  "US/Samoa",
   "UTC",
-  "Universal",
-  "W-SU",
-  "WET",
-  "Zulu",
   nullptr
 };
 
@@ -713,6 +609,22 @@ TEST(TimeZones, LoadZonesConcurrently) {
   EXPECT_LE(failures.size(), max_failures) << testing::PrintToString(failures);
 }
 
+TEST(TimeZone, UTC) {
+  const time_zone utc = utc_time_zone();
+
+  time_zone loaded_utc;
+  EXPECT_TRUE(load_time_zone("UTC", &loaded_utc));
+  EXPECT_EQ(loaded_utc, utc);
+
+  time_zone loaded_utc0;
+  EXPECT_TRUE(load_time_zone("UTC0", &loaded_utc0));
+  EXPECT_EQ(loaded_utc0, utc);
+
+  time_zone loaded_bad;
+  EXPECT_FALSE(load_time_zone("Invalid/TimeZone", &loaded_bad));
+  EXPECT_EQ(loaded_bad, utc);
+}
+
 TEST(TimeZone, NamedTimeZones) {
   const time_zone utc = utc_time_zone();
   EXPECT_EQ("UTC", utc.name());
@@ -832,7 +744,7 @@ TEST(BreakTime, LocalTimeInUTC) {
   const time_zone tz = utc_time_zone();
   const auto tp = chrono::system_clock::from_time_t(0);
   ExpectTime(tp, tz, 1970, 1, 1, 0, 0, 0, 0, false, "UTC");
-  EXPECT_EQ(weekday::thursday, get_weekday(civil_day(convert(tp, tz))));
+  EXPECT_EQ(weekday::thursday, get_weekday(convert(tp, tz)));
 }
 
 TEST(BreakTime, LocalTimeInUTCUnaligned) {
@@ -840,7 +752,7 @@ TEST(BreakTime, LocalTimeInUTCUnaligned) {
   const auto tp =
       chrono::system_clock::from_time_t(0) - chrono::milliseconds(500);
   ExpectTime(tp, tz, 1969, 12, 31, 23, 59, 59, 0, false, "UTC");
-  EXPECT_EQ(weekday::wednesday, get_weekday(civil_day(convert(tp, tz))));
+  EXPECT_EQ(weekday::wednesday, get_weekday(convert(tp, tz)));
 }
 
 TEST(BreakTime, LocalTimePosix) {
@@ -848,7 +760,7 @@ TEST(BreakTime, LocalTimePosix) {
   const time_zone tz = utc_time_zone();
   const auto tp = chrono::system_clock::from_time_t(536457599);
   ExpectTime(tp, tz, 1986, 12, 31, 23, 59, 59, 0, false, "UTC");
-  EXPECT_EQ(weekday::wednesday, get_weekday(civil_day(convert(tp, tz))));
+  EXPECT_EQ(weekday::wednesday, get_weekday(convert(tp, tz)));
 }
 
 TEST(TimeZoneImpl, LocalTimeInFixed) {
@@ -858,47 +770,47 @@ TEST(TimeZoneImpl, LocalTimeInFixed) {
   const auto tp = chrono::system_clock::from_time_t(0);
   ExpectTime(tp, tz, 1969, 12, 31, 15, 26, 13, offset.count(), false,
              "-083347");
-  EXPECT_EQ(weekday::wednesday, get_weekday(civil_day(convert(tp, tz))));
+  EXPECT_EQ(weekday::wednesday, get_weekday(convert(tp, tz)));
 }
 
 TEST(BreakTime, LocalTimeInNewYork) {
   const time_zone tz = LoadZone("America/New_York");
   const auto tp = chrono::system_clock::from_time_t(45);
   ExpectTime(tp, tz, 1969, 12, 31, 19, 0, 45, -5 * 60 * 60, false, "EST");
-  EXPECT_EQ(weekday::wednesday, get_weekday(civil_day(convert(tp, tz))));
+  EXPECT_EQ(weekday::wednesday, get_weekday(convert(tp, tz)));
 }
 
 TEST(BreakTime, LocalTimeInMTV) {
   const time_zone tz = LoadZone("America/Los_Angeles");
   const auto tp = chrono::system_clock::from_time_t(1380855729);
   ExpectTime(tp, tz, 2013, 10, 3, 20, 2, 9, -7 * 60 * 60, true, "PDT");
-  EXPECT_EQ(weekday::thursday, get_weekday(civil_day(convert(tp, tz))));
+  EXPECT_EQ(weekday::thursday, get_weekday(convert(tp, tz)));
 }
 
 TEST(BreakTime, LocalTimeInSydney) {
   const time_zone tz = LoadZone("Australia/Sydney");
   const auto tp = chrono::system_clock::from_time_t(90);
   ExpectTime(tp, tz, 1970, 1, 1, 10, 1, 30, 10 * 60 * 60, false, "AEST");
-  EXPECT_EQ(weekday::thursday, get_weekday(civil_day(convert(tp, tz))));
+  EXPECT_EQ(weekday::thursday, get_weekday(convert(tp, tz)));
 }
 
 TEST(MakeTime, TimePointResolution) {
   const time_zone utc = utc_time_zone();
   const time_point<chrono::nanoseconds> tp_ns =
       convert(civil_second(2015, 1, 2, 3, 4, 5), utc);
-  EXPECT_EQ("04:05", format("%M:%E*S", tp_ns, utc));
+  EXPECT_EQ("04:05", cctz::format("%M:%E*S", tp_ns, utc));
   const time_point<chrono::microseconds> tp_us =
       convert(civil_second(2015, 1, 2, 3, 4, 5), utc);
-  EXPECT_EQ("04:05", format("%M:%E*S", tp_us, utc));
+  EXPECT_EQ("04:05", cctz::format("%M:%E*S", tp_us, utc));
   const time_point<chrono::milliseconds> tp_ms =
       convert(civil_second(2015, 1, 2, 3, 4, 5), utc);
-  EXPECT_EQ("04:05", format("%M:%E*S", tp_ms, utc));
+  EXPECT_EQ("04:05", cctz::format("%M:%E*S", tp_ms, utc));
   const time_point<chrono::seconds> tp_s =
       convert(civil_second(2015, 1, 2, 3, 4, 5), utc);
-  EXPECT_EQ("04:05", format("%M:%E*S", tp_s, utc));
+  EXPECT_EQ("04:05", cctz::format("%M:%E*S", tp_s, utc));
   const time_point<cctz::seconds> tp_s64 =
       convert(civil_second(2015, 1, 2, 3, 4, 5), utc);
-  EXPECT_EQ("04:05", format("%M:%E*S", tp_s64, utc));
+  EXPECT_EQ("04:05", cctz::format("%M:%E*S", tp_s64, utc));
 
   // These next two require chrono::time_point_cast because the conversion
   // from a resolution of seconds (the return value of convert()) to a
@@ -906,11 +818,11 @@ TEST(MakeTime, TimePointResolution) {
   const time_point<chrono::minutes> tp_m =
       chrono::time_point_cast<chrono::minutes>(
           convert(civil_second(2015, 1, 2, 3, 4, 5), utc));
-  EXPECT_EQ("04:00", format("%M:%E*S", tp_m, utc));
+  EXPECT_EQ("04:00", cctz::format("%M:%E*S", tp_m, utc));
   const time_point<chrono::hours> tp_h =
       chrono::time_point_cast<chrono::hours>(
           convert(civil_second(2015, 1, 2, 3, 4, 5), utc));
-  EXPECT_EQ("00:00", format("%M:%E*S", tp_h, utc));
+  EXPECT_EQ("00:00", cctz::format("%M:%E*S", tp_h, utc));
 }
 
 TEST(MakeTime, Normalization) {
@@ -928,7 +840,7 @@ TEST(MakeTime, Normalization) {
 
 // NOTE: Run this with -ftrapv to detect overflow problems.
 TEST(MakeTime, SysSecondsLimits) {
-  const char RFC3339[] =  "%Y-%m-%dT%H:%M:%S%Ez";
+  const char RFC3339[] =  "%Y-%m-%d%ET%H:%M:%S%Ez";
   const time_zone utc = utc_time_zone();
   const time_zone east = fixed_time_zone(chrono::hours(14));
   const time_zone west = fixed_time_zone(-chrono::hours(14));
@@ -936,9 +848,11 @@ TEST(MakeTime, SysSecondsLimits) {
 
   // Approach the maximal time_point<cctz::seconds> value from below.
   tp = convert(civil_second(292277026596, 12, 4, 15, 30, 6), utc);
-  EXPECT_EQ("292277026596-12-04T15:30:06+00:00", format(RFC3339, tp, utc));
+  EXPECT_EQ("292277026596-12-04T15:30:06+00:00",
+            cctz::format(RFC3339, tp, utc));
   tp = convert(civil_second(292277026596, 12, 4, 15, 30, 7), utc);
-  EXPECT_EQ("292277026596-12-04T15:30:07+00:00", format(RFC3339, tp, utc));
+  EXPECT_EQ("292277026596-12-04T15:30:07+00:00",
+            cctz::format(RFC3339, tp, utc));
   EXPECT_EQ(time_point<cctz::seconds>::max(), tp);
   tp = convert(civil_second(292277026596, 12, 4, 15, 30, 8), utc);
   EXPECT_EQ(time_point<cctz::seconds>::max(), tp);
@@ -947,7 +861,8 @@ TEST(MakeTime, SysSecondsLimits) {
 
   // Checks that we can also get the maximal value for a far-east zone.
   tp = convert(civil_second(292277026596, 12, 5, 5, 30, 7), east);
-  EXPECT_EQ("292277026596-12-05T05:30:07+14:00", format(RFC3339, tp, east));
+  EXPECT_EQ("292277026596-12-05T05:30:07+14:00",
+            cctz::format(RFC3339, tp, east));
   EXPECT_EQ(time_point<cctz::seconds>::max(), tp);
   tp = convert(civil_second(292277026596, 12, 5, 5, 30, 8), east);
   EXPECT_EQ(time_point<cctz::seconds>::max(), tp);
@@ -956,7 +871,8 @@ TEST(MakeTime, SysSecondsLimits) {
 
   // Checks that we can also get the maximal value for a far-west zone.
   tp = convert(civil_second(292277026596, 12, 4, 1, 30, 7), west);
-  EXPECT_EQ("292277026596-12-04T01:30:07-14:00", format(RFC3339, tp, west));
+  EXPECT_EQ("292277026596-12-04T01:30:07-14:00",
+            cctz::format(RFC3339, tp, west));
   EXPECT_EQ(time_point<cctz::seconds>::max(), tp);
   tp = convert(civil_second(292277026596, 12, 4, 7, 30, 8), west);
   EXPECT_EQ(time_point<cctz::seconds>::max(), tp);
@@ -965,9 +881,11 @@ TEST(MakeTime, SysSecondsLimits) {
 
   // Approach the minimal time_point<cctz::seconds> value from above.
   tp = convert(civil_second(-292277022657, 1, 27, 8, 29, 53), utc);
-  EXPECT_EQ("-292277022657-01-27T08:29:53+00:00", format(RFC3339, tp, utc));
+  EXPECT_EQ("-292277022657-01-27T08:29:53+00:00",
+            cctz::format(RFC3339, tp, utc));
   tp = convert(civil_second(-292277022657, 1, 27, 8, 29, 52), utc);
-  EXPECT_EQ("-292277022657-01-27T08:29:52+00:00", format(RFC3339, tp, utc));
+  EXPECT_EQ("-292277022657-01-27T08:29:52+00:00",
+            cctz::format(RFC3339, tp, utc));
   EXPECT_EQ(time_point<cctz::seconds>::min(), tp);
   tp = convert(civil_second(-292277022657, 1, 27, 8, 29, 51), utc);
   EXPECT_EQ(time_point<cctz::seconds>::min(), tp);
@@ -976,7 +894,8 @@ TEST(MakeTime, SysSecondsLimits) {
 
   // Checks that we can also get the minimal value for a far-east zone.
   tp = convert(civil_second(-292277022657, 1, 27, 22, 29, 52), east);
-  EXPECT_EQ("-292277022657-01-27T22:29:52+14:00", format(RFC3339, tp, east));
+  EXPECT_EQ("-292277022657-01-27T22:29:52+14:00",
+            cctz::format(RFC3339, tp, east));
   EXPECT_EQ(time_point<cctz::seconds>::min(), tp);
   tp = convert(civil_second(-292277022657, 1, 27, 22, 29, 51), east);
   EXPECT_EQ(time_point<cctz::seconds>::min(), tp);
@@ -985,7 +904,8 @@ TEST(MakeTime, SysSecondsLimits) {
 
   // Checks that we can also get the minimal value for a far-west zone.
   tp = convert(civil_second(-292277022657, 1, 26, 18, 29, 52), west);
-  EXPECT_EQ("-292277022657-01-26T18:29:52-14:00", format(RFC3339, tp, west));
+  EXPECT_EQ("-292277022657-01-26T18:29:52-14:00",
+            cctz::format(RFC3339, tp, west));
   EXPECT_EQ(time_point<cctz::seconds>::min(), tp);
   tp = convert(civil_second(-292277022657, 1, 26, 18, 29, 51), west);
   EXPECT_EQ(time_point<cctz::seconds>::min(), tp);
@@ -999,13 +919,23 @@ TEST(MakeTime, SysSecondsLimits) {
 #if defined(_WIN32) || defined(_WIN64)
     // localtime_s() and gmtime_s() don't believe in years outside [1970:3000].
 #else
-    const time_zone utc = LoadZone("libc:UTC");
+    const time_zone cut = LoadZone("libc:UTC");
     const year_t max_tm_year = year_t{std::numeric_limits<int>::max()} + 1900;
-    tp = convert(civil_second(max_tm_year, 12, 31, 23, 59, 59), utc);
-    EXPECT_EQ("2147485547-12-31T23:59:59+00:00", format(RFC3339, tp, utc));
+    tp = convert(civil_second(max_tm_year, 12, 31, 23, 59, 59), cut);
+#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__EMSCRIPTEN__)
+    // Some gmtime_r() impls fail on extreme positive values.
+#else
+    EXPECT_EQ("2147485547-12-31T23:59:59+00:00",
+              cctz::format(RFC3339, tp, cut));
+#endif
     const year_t min_tm_year = year_t{std::numeric_limits<int>::min()} + 1900;
-    tp = convert(civil_second(min_tm_year, 1, 1, 0, 0, 0), utc);
-    EXPECT_EQ("-2147481748-01-01T00:00:00+00:00", format(RFC3339, tp, utc));
+    tp = convert(civil_second(min_tm_year, 1, 1, 0, 0, 0), cut);
+#if defined(__Fuchsia__) || defined(__EMSCRIPTEN__)
+    // Some gmtime_r() impls fail on extreme negative values (fxbug.dev/78527).
+#else
+    EXPECT_EQ("-2147481748-01-01T00:00:00+00:00",
+              cctz::format(RFC3339, tp, cut));
+#endif
 #endif
   }
 }
@@ -1017,24 +947,24 @@ TEST(MakeTime, LocalTimeLibC) {
   //  1) we know how to change the time zone used by localtime()/mktime(),
   //  2) cctz and localtime()/mktime() will use similar-enough tzdata, and
   //  3) we have some idea about how mktime() behaves during transitions.
-#if defined(__linux__) && !defined(__ANDROID__)
+#if defined(__linux__) && defined(__GLIBC__) && !defined(__ANDROID__)
   const char* const ep = getenv("TZ");
   std::string tz_name = (ep != nullptr) ? ep : "";
   for (const char* const* np = kTimeZoneNames; *np != nullptr; ++np) {
     ASSERT_EQ(0, setenv("TZ", *np, 1));  // change what "localtime" means
     const auto zi = local_time_zone();
     const auto lc = LoadZone("libc:localtime");
-    time_zone::civil_transition trans;
+    time_zone::civil_transition transition;
     for (auto tp = zi.lookup(civil_second()).trans;
-         zi.next_transition(tp, &trans);
-         tp = zi.lookup(trans.to).trans) {
-      const auto fcl = zi.lookup(trans.from);
-      const auto tcl = zi.lookup(trans.to);
-      civil_second cs;  // compare cs in zi and lc
+         zi.next_transition(tp, &transition);
+         tp = zi.lookup(transition.to).trans) {
+      const auto fcl = zi.lookup(transition.from);
+      const auto tcl = zi.lookup(transition.to);
+      civil_second cs, us;  // compare cs and us in zi and lc
       if (fcl.kind == time_zone::civil_lookup::UNIQUE) {
         if (tcl.kind == time_zone::civil_lookup::UNIQUE) {
           // Both unique; must be an is_dst or abbr change.
-          ASSERT_EQ(trans.from, trans.to);
+          ASSERT_EQ(transition.from, transition.to);
           const auto trans = fcl.trans;
           const auto tal = zi.lookup(trans);
           const auto tprev = trans - cctz::seconds(1);
@@ -1045,13 +975,15 @@ TEST(MakeTime, LocalTimeLibC) {
           continue;
         }
         ASSERT_EQ(time_zone::civil_lookup::REPEATED, tcl.kind);
-        cs = trans.to;
+        cs = transition.to;
+        us = transition.from;
       } else {
         ASSERT_EQ(time_zone::civil_lookup::UNIQUE, tcl.kind);
         ASSERT_EQ(time_zone::civil_lookup::SKIPPED, fcl.kind);
-        cs = trans.from;
+        cs = transition.from;
+        us = transition.to;
       }
-      if (cs.year() > 2037) break;  // limit test time (and to 32-bit time_t)
+      if (us.year() > 2037) break;  // limit test time (and to 32-bit time_t)
       const auto cl_zi = zi.lookup(cs);
       if (zi.lookup(cl_zi.pre).is_dst == zi.lookup(cl_zi.post).is_dst) {
         // The "libc" implementation cannot correctly classify transitions
@@ -1083,6 +1015,13 @@ TEST(MakeTime, LocalTimeLibC) {
       EXPECT_EQ(cl_zi.pre, cl_lc.pre);
       EXPECT_EQ(cl_zi.trans, cl_lc.trans);
       EXPECT_EQ(cl_zi.post, cl_lc.post);
+      const auto ucl_zi = zi.lookup(us);
+      const auto ucl_lc = lc.lookup(us);
+      SCOPED_TRACE(testing::Message() << "For " << us << " in " << *np);
+      EXPECT_EQ(ucl_zi.kind, ucl_lc.kind);
+      EXPECT_EQ(ucl_zi.pre, ucl_lc.pre);
+      EXPECT_EQ(ucl_zi.trans, ucl_lc.trans);
+      EXPECT_EQ(ucl_zi.post, ucl_lc.post);
     }
   }
   if (ep == nullptr) {
@@ -1156,6 +1095,45 @@ TEST(PrevTransition, AmericaNewYork) {
   // We have a transition but we don't know which one.
 }
 
+TEST(NextTransition, Scan) {
+  for (const char* const* np = kTimeZoneNames; *np != nullptr; ++np) {
+    SCOPED_TRACE(testing::Message() << "In " << *np);
+    time_zone tz;
+    // EXPECT_TRUE(load_time_zone(*np, &tz));
+    if (!load_time_zone(*np, &tz)) {
+      continue;  // tolerate kTimeZoneNames/zoneinfo skew
+    }
+
+    auto tp = time_point<cctz::seconds>::min();
+    time_zone::civil_transition trans;
+    while (tz.next_transition(tp, &trans)) {
+      time_zone::civil_lookup from_cl = tz.lookup(trans.from);
+      EXPECT_NE(from_cl.kind, time_zone::civil_lookup::REPEATED);
+      time_zone::civil_lookup to_cl = tz.lookup(trans.to);
+      EXPECT_NE(to_cl.kind, time_zone::civil_lookup::SKIPPED);
+
+      auto trans_tp = to_cl.trans;
+      time_zone::absolute_lookup trans_al = tz.lookup(trans_tp);
+      EXPECT_EQ(trans_al.cs, trans.to);
+      auto pre_trans_tp = trans_tp - cctz::seconds(1);
+      time_zone::absolute_lookup pre_trans_al = tz.lookup(pre_trans_tp);
+      EXPECT_EQ(pre_trans_al.cs + 1, trans.from);
+
+      auto offset_delta = trans_al.offset - pre_trans_al.offset;
+      EXPECT_EQ(offset_delta, trans.to - trans.from);
+      if (offset_delta == 0) {
+        // This "transition" is only an is_dst or abbr change.
+        EXPECT_EQ(to_cl.kind, time_zone::civil_lookup::UNIQUE);
+        if (trans_al.is_dst == pre_trans_al.is_dst) {
+          EXPECT_STRNE(trans_al.abbr, pre_trans_al.abbr);
+        }
+      }
+
+      tp = trans_tp;  // continue scan from transition
+    }
+  }
+}
+
 TEST(TimeZoneEdgeCase, AmericaNewYork) {
   const time_zone tz = LoadZone("America/New_York");
 
@@ -1270,10 +1248,10 @@ TEST(TimeZoneEdgeCase, PacificApia) {
   //   1325239200 == Sat, 31 Dec 2011 00:00:00 +1400 (+14)
   auto tp = convert(civil_second(2011, 12, 29, 23, 59, 59), tz);
   ExpectTime(tp, tz, 2011, 12, 29, 23, 59, 59, -10 * 3600, true, "-10");
-  EXPECT_EQ(363, get_yearday(civil_day(convert(tp, tz))));
+  EXPECT_EQ(363, get_yearday(convert(tp, tz)));
   tp += cctz::seconds(1);
   ExpectTime(tp, tz, 2011, 12, 31, 0, 0, 0, 14 * 3600, true, "+14");
-  EXPECT_EQ(365, get_yearday(civil_day(convert(tp, tz))));
+  EXPECT_EQ(365, get_yearday(convert(tp, tz)));
 }
 
 TEST(TimeZoneEdgeCase, AfricaCairo) {
@@ -1395,10 +1373,10 @@ TEST(TimeZoneEdgeCase, NegativeYear) {
   const time_zone tz = utc_time_zone();
   auto tp = convert(civil_second(0, 1, 1, 0, 0, 0), tz);
   ExpectTime(tp, tz, 0, 1, 1, 0, 0, 0, 0 * 3600, false, "UTC");
-  EXPECT_EQ(weekday::saturday, get_weekday(civil_day(convert(tp, tz))));
+  EXPECT_EQ(weekday::saturday, get_weekday(convert(tp, tz)));
   tp -= cctz::seconds(1);
   ExpectTime(tp, tz, -1, 12, 31, 23, 59, 59, 0 * 3600, false, "UTC");
-  EXPECT_EQ(weekday::friday, get_weekday(civil_day(convert(tp, tz))));
+  EXPECT_EQ(weekday::friday, get_weekday(convert(tp, tz)));
 }
 
 TEST(TimeZoneEdgeCase, UTC32bitLimit) {
